from django.contrib.auth.hashers import check_password, make_password
from usermerge.models import Admin, Registry, User

# Create your backends here.
# https://docs.djangoproject.com/en/2.0/topics/auth/customizing/#authentication-backends

# For more information on default/customizing user authentication and password management, see https://docs.djangoproject.com/en/2.0/topics/auth/ .
# For more information on the platform names that are provided in the drop-down list of the login form, see login.html and get_names_of_SoftLab_provided_platforms_from_DB() function of views.py .

class UserBackend:
    def authenticate(self, request, platform = None, username = None, password = None):
        """
        Authenticate the credentials POSTed via the login form.

        If the selected platform is SLUB, the user should be authenticated as Admin with the happy path being:
        * Try to find an Admin instance of usermergeDB that corresponds to the provided username.
        * If a corresponding instance is found, check if the provided password matches the one of the instance.
        * If the provided password matches the one of the instance, the user is authenticated successfully as Admin and
          the instance is returned.

        If the selected platform is not SLUB, the user should be authenticated as User with the happy path being:
        * Try to find a Registry instance of usermergeDB that corresponds to the provided platform and username.
        * If a corresponding instance is found, check if the provided password matches the one of the instance.
        * If the provided password matches the one of the instance, the user is authenticated successfully as User and
          the User instance that is associated with the corresponding Registry instance is returned.

        If any of the checks (finding the corresponding instance and comparing the passwords) fails, the authentication
        fails and an empty instance (None) is implicitly returned (if the first check fails, the second is never performed).
        """
        if platform == 'SLUB':
            # Admin authentication
            try:
                admin = Admin.objects.get(username = username)
            except Admin.DoesNotExist:
                # Run the default password hasher once to reduce the timing difference between an existent and
                # a non-existent Admin instance (#20760), as done in ModelBackend's authenticate() method
                # (https://github.com/django/django/blob/master/django/contrib/auth/backends.py).
                dummy_password = make_password(password)
            else:
                if check_password(password, admin.password):
                    return admin
        else:
            # User authentication
            try:
                registry = Registry.objects.get(platform__name = platform, username = username)
            except Registry.DoesNotExist:
                # Run the default password hasher once to reduce the timing difference between an existent and
                # a non-existent User instance (#20760), as done in ModelBackend's authenticate() method
                # (https://github.com/django/django/blob/master/django/contrib/auth/backends.py).
                dummy_password = make_password(password)
            else:
                if check_password(password, registry.password):
                    return registry.user

    def get_user(self, user_model_label, user_id):
        """
        Return the user instance associated with the given model and id (if model is neither User nor Admin, raise a ValueError exception).
        If no user is retrieved, return an empty instance (None).
        """
        if user_model_label == 'usermerge.User':
            try:
                return User.objects.get(pk = user_id)
            except User.DoesNotExist:
                return None
        elif user_model_label == 'usermerge.Admin':
            try:
                return Admin.objects.get(pk = user_id)
            except Admin.DoesNotExist:
                return None
        else:
            raise ValueError('The user_model_label argument of get_user() backend method '
                             'should either be "usermerge.User" or "usermerge.Admin"!')

